@@ -15,19 +15,21 @@ if(getRversion() >= "2.15.1")  utils::globalVariables(c("."))
 #' @param network An object of class `mbnma.network`.
 #' @param parameters.to.save A character vector containing names of parameters
 #'   to monitor in JAGS
-#' @param fun A string specifying a functional form to be assigned to the
+#' @param fun A character vector specifying a functional form to be assigned to the
 #'   dose-response. Options are given in `details`.
-#' @param user.fun A string specifying any relationship including `dose` and
-#'   one/several of: `beta.1`, `beta.2`, `beta.3`.
+#' @param user.fun A formula specifying any relationship including `dose` and
+#'   one/several of: `beta.1`, `beta.2`, `beta.3`, `beta.4`.
 #' @param model.file A JAGS model written as a character object that can be used
 #'   to overwrite the JAGS model that is automatically written based on the
 #'   specified options.
 #'
-#' @param beta.1 Refers to dose-parameter(s) specified within the dose-response function.
+#' @param beta.1 Refers to dose-parameter(s) specified within the dose-response function(s).
 #' Can take either `"rel"`, `"common"`, `"random"`, or be assigned a numeric value (see details).
-#' @param beta.2 Refers to dose-parameter(s) specified within the dose-response function.
+#' @param beta.2 Refers to dose-parameter(s) specified within the dose-response function(s).
 #' Can take either `"rel"`, `"common"`, `"random"`, or be assigned a numeric value (see details).
-#' @param beta.3 Refers to dose-parameter(s) specified within the dose-response function.
+#' @param beta.3 Refers to dose-parameter(s) specified within the dose-response function(s).
+#' Can take either `"rel"`, `"common"`, `"random"`, or be assigned a numeric value (see details).
+#' @param beta.4 Refers to dose-parameter(s) specified within the dose-response function(s).
 #' Can take either `"rel"`, `"common"`, `"random"`, or be assigned a numeric value (see details).
 #'
 #' @param method Can take either `"common"` or `"random"` to indicate whether relative effects
@@ -99,7 +101,7 @@ if(getRversion() >= "2.15.1")  utils::globalVariables(c("."))
 #'   (e.g. `d.ed50` or `d.1`):
 #'   * `d` The pooled effect for each agent for a given dose-response
 #'   parameter. Will be estimated by the model if dose-response parameters (`beta.1`,
-#'   `beta.2`, `beta.3`) are set to `"rel"`.
+#'   `beta.2`, `beta.3`, `beta.4`) are set to `"rel"`.
 #'   * `sd` (without a suffix) - the between-study SD (heterogeneity) for relative effects, reported if
 #'   `method="random"`.
 #'   * `D` The class effect for each class for a given dose-response
@@ -109,10 +111,10 @@ if(getRversion() >= "2.15.1")  utils::globalVariables(c("."))
 #'   set to `"random"`.
 #'   * `beta` The absolute value of a given dose-response parameter across the whole
 #'   network (does not vary by agent/class). Will be estimated by the model if
-#'   dose-response parameters (`beta.1`, `beta.2`, `beta.3`) are set to `"common"`
+#'   dose-response parameters (`beta.1`, `beta.2`, `beta.3`, `beta.4`) are set to `"common"`
 #'   or `"random"`.
 #'   * `sd` (with a suffix) - the between-study SD (heterogeneity) for absolute dose-response
-#'   parameters, reported if `beta.1`, `beta.2` or `beta.3` are set to `"random"`
+#'   parameters, reported if `beta.1`, `beta.2`, `beta.3` or `beta.4` are set to `"random"`
 #'   * `totresdev` The residual deviance of the model
 #'   * `deviance` The deviance of the model
 #'
@@ -125,7 +127,7 @@ if(getRversion() >= "2.15.1")  utils::globalVariables(c("."))
 #'
 #' @section Dose-response parameters:
 #' * `"rel"` implies that relative effects should be pooled for this dose-response
-#' parameter, that vary by agent.
+#' parameter separately for each agent in the network.
 #' * `"common"` implies that all studies estimate the same true absolute effect
 #' (akin to a "fixed effects" meta-analysis) across the whole network
 #' * `"random"` implies that all studies estimate a separate true absolute effect, but
@@ -141,7 +143,7 @@ if(getRversion() >= "2.15.1")  utils::globalVariables(c("."))
 #'   Several general dose-response functions are provided, but a
 #'   user-defined dose-response relationship can instead be used.
 #'
-#'   Built-in time-course functions are:
+#'   Built-in dose-response functions are:
 #'   * `"linear"`: `beta.1` refers to the gradient
 #'   * `"exponential"`: `beta.1` refers to the rate of gain/decay
 #'   * `"emax"` (emax without a Hill parameter): `beta.1` refers to
@@ -154,6 +156,20 @@ if(getRversion() >= "2.15.1")  utils::globalVariables(c("."))
 #'   the method of \insertCite{owen2015;textual}{MBNMAdose})
 #'   * `"user"` (user-defined function: `user.fun` must be specified in arguments)
 #'
+#'
+#'   As of version 0.2.5, separate dose-response functions can be specified for
+#'   different agents in the network by passing a character vector with multiple elements to `fun`.
+#'   Each agent in `network` is assigned the dose-response function in the corresponding element in `fun`.
+#'   `fun` must therefore be the same length as the number of agents in `network`. Dose-response parameters
+#'   `beta.1`, `beta.2`, `beta.3` and `beta.4` refer to the corresponding dose-response parameters across
+#'   the multiple functions in the following order: `user`, `linear`, `exponential`, `emax`, `emax.hill`.
+#'
+#'   This would mean that if `fun` included `linear`, `exponential` and `emax` within it then for the
+#'   corresponding agents `beta.1`
+#'   would refer to linear slope parameters, `beta.2` to exponential rate of growth/decay parameters,
+#'   `beta.3` to Emax parameters, and `beta.4` to ED50 parameters.
+#'
+#'
 #' @importFrom Rdpack reprompt
 #' @importFrom magrittr "%>%"
 #'
@@ -175,7 +191,7 @@ if(getRversion() >= "2.15.1")  utils::globalVariables(c("."))
 #' result <- mbnma.run(network, fun="exponential", beta.1="rel", method="random")
 #'
 #' # Fit a user-defined function (quadratic)
-#' fun.def <- "(beta.1 * dose) + (beta.2 * (dose^2))"
+#' fun.def <- ~ (beta.1 * dose) + (beta.2 * (dose^2))
 #' result <- mbnma.run(network, fun="user", user.fun=fun.def,
 #'               beta.1="rel", beta.2="rel", method="common")
 #'
@@ -190,6 +206,10 @@ if(getRversion() >= "2.15.1")  utils::globalVariables(c("."))
 #' result <- mbnma.run(network, fun="emax.hill",
 #'               beta.1="rel", beta.2="rel", beta.3=5, method="random")
 #'
+#' # Fit a model with different dose-response functions for each agent
+#' multidose <- mbnma.run(network, fun=c("emax", "emax", "emax", "exponential",
+#'                  "emax", "emax", "exponential", "emax"))
+#'
 #'
 #' ########## Class effects ##########
 #'
@@ -262,14 +282,14 @@ if(getRversion() >= "2.15.1")  utils::globalVariables(c("."))
 mbnma.run <- function(network,
                       fun="linear",
                       beta.1="rel",
-                      beta.2=NULL, beta.3=NULL,
+                      beta.2="rel", beta.3="rel", beta.4="rel",
                       method="common",
                       class.effect=list(),
                       cor=TRUE,
                       var.scale=NULL,
                       user.fun=NULL,
                       parameters.to.save=NULL,
-                      pd="pv", parallel=TRUE,
+                      pd="pv", parallel=FALSE,
                       likelihood=NULL, link=NULL,
                       priors=NULL,
                       model.file=NULL,
@@ -304,12 +324,13 @@ mbnma.run <- function(network,
     plac.incl <- TRUE
   } else {
     plac.incl <- FALSE
-    if (fun %in% c("nonparam.up", "nonparam.down")) {
-      stop("Placebo (or an agent with dose=0) must be included in the network to model a nonparametric dose-response relationship")
-    }
+    # if (any(c("nonparam.up", "nonparam.down") %in% fun)) {
+    #   stop("Placebo (or an agent with dose=0) must be included in the network to model a nonparametric dose-response relationship")
+    # }
   }
 
 
+  assigned.class <- class.effect
   if (!is.null(arg.params)) {
     if (!all((names(arg.params)) %in% c("wrap.params", "run.params"))) {
       stop("arg.params has been incorrectly specified")
@@ -324,20 +345,27 @@ mbnma.run <- function(network,
     names(class.effect) <- fun.params
 
   } else if (is.null(arg.params)) {
-    wrap.params <- list(beta.1, beta.2, beta.3)
+    wrap.params <- list(beta.1, beta.2, beta.3, beta.4)
     wrap.params <- which(sapply(wrap.params,
                                 is.character))
   }
 
   if (is.null(model.file)) {
     model <- mbnma.write(fun=fun, user.fun=user.fun,
-                         beta.1=beta.1, beta.2=beta.2, beta.3=beta.3,
+                         beta.1=beta.1, beta.2=beta.2, beta.3=beta.3, beta.4=beta.4,
                          method=method,
                          class.effect=class.effect,
                          cor=cor, var.scale=var.scale,
                          likelihood=likelihood, link=link
     )
 
+    # Edit beta parameters if they aren't in dose-response function
+    for (i in 1:4) {
+      if (!(grepl(paste0("beta\\.", i), model) | grepl(paste0("d\\.", i), model))) {
+        assign(paste0("beta.", i), NULL)
+      }
+    }
+
     # Change code for if plac not included in network
     if (plac.incl==FALSE) {
       model <- gsub("\\\nfor \\(k in 2:Nagent\\)\\{ # Priors on relative treatment effects\\\n",
@@ -347,7 +375,7 @@ mbnma.run <- function(network,
                     "for (k in 1:Nclass){ # Priors on relative class effects\n",
                     model)
 
-      model <- gsub("s\\.beta\\.[1-3]\\[1\\] <- 0", "", model)
+      model <- gsub("s\\.beta\\.[1-4]\\[1\\] <- 0", "", model)
     }
 
     # Change beta.1 and beta.2 to emax and et50, etc. if necessary
@@ -361,11 +389,11 @@ mbnma.run <- function(network,
         }
       }
 
-      wrap.params <- wrap.params[which(sapply(list(beta.1, beta.2, beta.3),
+      wrap.params <- wrap.params[which(sapply(list(beta.1, beta.2, beta.3, beta.4),
                                               is.character))]
 
     } else {
-      wrap.params <- which(sapply(list(beta.1, beta.2, beta.3),
+      wrap.params <- which(sapply(list(beta.1, beta.2, beta.3, beta.4),
                                   is.character))
     }
 
@@ -400,10 +428,9 @@ mbnma.run <- function(network,
     class <- TRUE
   } else {class <- FALSE}
 
-
   #### Run jags model ####
 
-  if (fun %in% c("nonparam.up", "nonparam.down")) {
+  if (any(c("nonparam.up", "nonparam.down") %in% fun)) {
     # Change doses to dose indices
     data.ab <- index.dose(network[["data.ab"]])[["data.ab"]]
   } else {
@@ -413,11 +440,12 @@ mbnma.run <- function(network,
   result.jags <- mbnma.jags(data.ab, model,
                             class=class,
                             parameters.to.save=parameters.to.save,
-                            likelihood=likelihood, link=link,
+                            likelihood=likelihood, link=link, fun=fun,
                             n.iter=n.iter,
                             n.thin=n.thin,
                             n.chains=n.chains,
                             n.burnin=n.burnin,
+                            parallel=parallel,
                             ...)
   result <- result.jags[["jagsoutput"]]
   jagsdata <- result.jags[["jagsdata"]]
@@ -448,7 +476,7 @@ mbnma.run <- function(network,
   }
 
   # Recalculate DIC so it is adjusted for choice of pD
-  result$DIC <- result$BUGSoutput$pD + result$BUGSoutput$median$deviance
+  result$BUGSoutput$DIC <- result$BUGSoutput$pD + result$BUGSoutput$median$deviance
 
   # Add variables for other key model characteristics (for predict and plot functions)
   model.arg <- list("parameters.to.save"=assigned.parameters.to.save,
@@ -456,21 +484,29 @@ mbnma.run <- function(network,
                     "jagscode"=model,
                     "beta.1"=beta.1, "beta.2"=beta.2,
                     "beta.3"=beta.3,
+                    "beta.4"=beta.4,
                     "method"=method,
                     "likelihood"=likelihood, "link"=link,
-                    "class.effect"=class.effect,
+                    #"class.effect"=class.effect,
+                    "class.effect"=assigned.class,
                     "cor"=cor,
                     "var.scale"=var.scale,
                     "parallel"=parallel, "pd"=pd,
                     "priors"=get.prior(model), "arg.params"=arg.params)
   result[["model.arg"]] <- model.arg
   result[["type"]] <- "dose"
-  result[["agents"]] <- network[["agents"]]
-  result[["treatments"]] <- network[["treatments"]]
+  result[["network"]] <- network
+  #result[["agents"]] <- network[["agents"]]
+  #result[["treatments"]] <- network[["treatments"]]
   if (length(class.effect)>0) {
     result[["classes"]] <- network[["classes"]]
   }
 
+  # Remove dose-response parameters for agents if multi dr functions are used
+  if (!is.list(result)) {
+    result <- cutjags(result)
+  }
+
   if (!("error" %in% names(result))) {
     class(result) <- c("mbnma", class(result))
   }
@@ -485,19 +521,23 @@ mbnma.run <- function(network,
 
 mbnma.jags <- function(data.ab, model,
                        class=FALSE,
-                       parameters.to.save=parameters.to.save,
-                       likelihood=NULL, link=NULL,
-                       warn.rhat=FALSE, ...) {
+                       #parameters.to.save=parameters.to.save,
+                       likelihood=NULL, link=NULL, fun=NULL,
+                       warn.rhat=FALSE, parallel=FALSE, ...) {
 
   # Run checks
   argcheck <- checkmate::makeAssertCollection()
   checkmate::assertDataFrame(data.ab, add=argcheck)
   checkmate::assertCharacter(model, any.missing=FALSE, len=1, add=argcheck)
+  checkmate::assertLogical(parallel, len=1, null.ok=FALSE, any.missing=FALSE, add=argcheck)
   checkmate::assertLogical(class, len=1, null.ok=FALSE, any.missing=FALSE, add=argcheck)
-  checkmate::assertCharacter(parameters.to.save, any.missing=FALSE, unique=TRUE,
+  #checkmate::assertCharacter(parameters.to.save, any.missing=FALSE, unique=TRUE,
+  #                           null.ok=TRUE, add=argcheck)
+  checkmate::assertCharacter(fun, any.missing=FALSE,
                              null.ok=TRUE, add=argcheck)
   checkmate::reportAssertions(argcheck)
 
+  args <- list(...)
 
   if (is.null(likelihood) & is.null(link)) {
     # For MBNMAtime
@@ -506,7 +546,7 @@ mbnma.jags <- function(data.ab, model,
   } else {
     # For MBNMAdose
     jagsdata <- getjagsdata(data.ab, class=class,
-                            likelihood=likelihood, link=link) # get data into jags correct format
+                            likelihood=likelihood, link=link, fun=fun) # get data into jags correct format
   }
 
 
@@ -516,6 +556,18 @@ mbnma.jags <- function(data.ab, model,
   } else if (grepl("maxdose", model)) {
     #jagsdata[["maxdose"]] <- index.dose(network[["data.ab"]])[["maxdose"]]
     jagsdata[["maxdose"]] <- index.dose(data.ab)[["maxdose"]]
+
+    # Generate monotonically increasing/decreasing initial values
+    # Check for user-defined initial values
+    if (!("inits" %in% names(args))) {
+      if (grepl("T\\(d\\.1\\[c-1,k\\],\\)", model)) {
+        fun="nonparam.up"
+      } else if (grepl("T\\(,d\\.1\\[c-1,k\\]\\)", model)) {
+        fun=="nonparam.down"
+      }
+
+      args$inits <- gen.inits(jagsdata, fun=fun, n.chains=args$n.chains)
+    }
   }
 
   # Put data from jagsdata into separate R objects
@@ -538,11 +590,14 @@ mbnma.jags <- function(data.ab, model,
   cat(model,file=tmps)
   close(tmps)
 
+  #print(args)
   out <- tryCatch({
-    result <- R2jags::jags(data=jagsvars, model.file=tmpf,
-                           parameters.to.save=parameters.to.save,
-                           ...
-    )
+    if (parallel==FALSE) {
+      result <- do.call(R2jags::jags, c(args, list(data=jagsvars, model.file=tmpf)))
+    } else if (parallel==TRUE) {
+      result <- do.call(R2jags::jags.parallel, c(args, list(data=jagsvars, model.file=tmpf)))
+      class(result) <- class(result)[c(2,1)]
+    }
   },
   error=function(cond) {
     message(cond)
@@ -563,6 +618,41 @@ mbnma.jags <- function(data.ab, model,
 
 
 
+#' Generate initial values for non-parametric dose-response functions
+#'
+#' Ensures model runs properly
+#'
+#' @noRd
+gen.inits <- function(jagsdata, fun, n.chains) {
+  if ("maxdose" %in% names(jagsdata)) {
+    inits <- list()
+    for (i in 1:n.chains) {
+      inits[[length(inits)+1]] <- list("d.1"=gen.init(jagsdata, fun))
+    }
+  }
+  return(inits)
+}
+
+
+
+
+
+gen.init <- function(jagsdata, fun) {
+  sapply(jagsdata$maxdose, FUN=function(x, direction=fun) {
+   val <- x-1
+
+   if (direction=="nonparam.up") {
+     start <- stats::runif(1,0,3)
+     c(NA, seq(start, start+stats::runif(1,0,3), length.out=val), rep(NA, max(jagsdata$maxdose)-(val+1)))
+   } else if (direction=="nonparam.down") {
+     start <- stats::runif(1,-3,0)
+     c(NA, seq(start, start-stats::runif(1,0,3), length.out=val), rep(NA, max(jagsdata$maxdose)-(val+1)))
+   }
+  })
+}
+
+
+
 
 #' Automatically generate parameters to save for a dose-response MBNMA model
 #'
@@ -687,6 +777,8 @@ nma.run <- function(network, method="common", likelihood=NULL, link=NULL,
   checkmate::assertLogical(UME, add=argcheck)
   checkmate::reportAssertions(argcheck)
 
+  args <- list(...)
+
   # Check/assign link and likelihood
   likelink <- check.likelink(network$data.ab, likelihood=likelihood, link=link)
   likelihood <- likelink[["likelihood"]]
@@ -743,11 +835,8 @@ nma.run <- function(network, method="common", likelihood=NULL, link=NULL,
   close(tmps)
 
   out <- tryCatch({
-    result <- R2jags::jags(data=jagsvars, model.file=tmpf,
-                           parameters.to.save=parameters.to.save,
-                           n.iter=n.iter,
-                           ...
-    )
+    result <- do.call(R2jags::jags, c(args, list(data=jagsvars, model.file=tmpf, n.iter=n.iter,
+                                                 parameters.to.save=parameters.to.save)))
   },
   error=function(cond) {
     message(cond)
@@ -931,7 +1020,7 @@ mbnma.linear <- function(network,
                          cor=TRUE,
                          var.scale=NULL,
                          parameters.to.save=NULL,
-                         pd="pv", parallel=TRUE,
+                         pd="pv", parallel=FALSE,
                          likelihood=NULL, link=NULL,
                          priors=NULL,
                          arg.params=NULL, ...)
@@ -1048,7 +1137,7 @@ mbnma.exponential <- function(network,
                          cor=TRUE,
                          var.scale=NULL,
                          parameters.to.save=NULL,
-                         pd="pv", parallel=TRUE,
+                         pd="pv", parallel=FALSE,
                          likelihood=NULL, link=NULL,
                          priors=NULL,
                          arg.params=NULL, ...)
@@ -1196,7 +1285,7 @@ mbnma.emax <- function(network,
                          cor=TRUE,
                          var.scale=NULL,
                          parameters.to.save=NULL,
-                         pd="pv", parallel=TRUE,
+                         pd="pv", parallel=FALSE,
                          likelihood=NULL, link=NULL,
                          priors=NULL,
                          arg.params=NULL, ...)
@@ -1346,7 +1435,7 @@ mbnma.emax.hill <- function(network,
                        cor=TRUE,
                        var.scale=NULL,
                        parameters.to.save=NULL,
-                       pd="pv", parallel=TRUE,
+                       pd="pv", parallel=FALSE,
                        likelihood=NULL, link=NULL,
                        priors=NULL,
                        arg.params=NULL, ...)
@@ -1354,7 +1443,7 @@ mbnma.emax.hill <- function(network,
 
   arg.params <- list(
     wrap.params=c("emax", "ed50", "hill"),
-    run.params=c("beta.1", "beta.2", "beta.3")
+    run.params=c("beta.1", "beta.2", "beta.3", "beta.4")
   )
 
   result <- mbnma.run(network=network, parameters.to.save=parameters.to.save,
